@@ -3,14 +3,15 @@ package CGI::Wiki;
 use strict;
 
 use vars qw( $VERSION );
-$VERSION = '0.14';
+$VERSION = '0.15';
 
 use CGI ":standard";
 use Carp qw(croak carp);
 use Digest::MD5 "md5_hex";
 use Class::Delegation
     send => ['retrieve_node', 'retrieve_node_and_checksum', 'verify_checksum',
-             'list_all_nodes', 'list_recent_changes', 'node_exists'],
+             'list_all_nodes', 'list_recent_changes', 'node_exists',
+             'list_backlinks'],
     to   => '_store',
     send => 'delete_node',
     to   => ['_store', '_search'],
@@ -28,7 +29,14 @@ Helps you develop Wikis quickly by taking care of the boring bits for
 you. The aim is to allow different types of backend storage and search
 without you having to worry about the details.
 
-=head1 IMPORTANT NOTE
+=head1 IMPORTANT NOTE WHEN UPGRADING FROM PRE-0.15 VERSIONS
+
+The database schema changed between versions 0.14 and 0.15 - see the
+'Changes' file for details. This is really kinda important, please do
+check this out or your code will die when it tries to use any existing
+databases.
+
+=head1 NOTE WHEN UPGRADING FROM PRE-0.10 VERSIONS
 
 There was a small interface change between versions 0.05 and 0.10 -
 see the 'Changes' file for details.
@@ -202,10 +210,21 @@ sub write_node {
     croak "No content parameter supplied for writing" unless defined $content;
     $checksum = md5_hex("") unless defined $checksum;
 
+    my $formatter = $self->{_formatter};
+    my @links_to;
+    if ( $formatter->can( "find_internal_links" ) ) {
+        my @all_links_to = $formatter->find_internal_links( $content );
+        my %unique = map { $_ => 1 } @all_links_to;
+        @links_to = keys %unique;
+    }
+
+    my %data = ( node     => $node,
+		 content  => $content,
+		 checksum => $checksum );
+    $data{links_to} = \@links_to if scalar @links_to;
+
     my $store = $self->store;
-    $store->check_and_write_node( node     => $node,
-				  content  => $content,
-				  checksum => $checksum ) or return 0;
+    $store->check_and_write_node( %data ) or return 0;
 
     my $search = $self->{_search};
     if ($search) {
@@ -271,6 +290,8 @@ backend, if any)
 
 =item * list_all_nodes
 
+=item * list_backlinks
+
 =item * list_recent_changes
 
 =item * node_exists
@@ -341,6 +362,8 @@ Other ways to implement Wikis in Perl include:
 
 =item * L<AxKit::XSP::Wiki>
 
+=item * L<Apache::MiniWiki>
+
 =item * UseModWiki
 
 =back
@@ -351,7 +374,7 @@ Kake Pugh (kake@earth.li).
 
 =head1 COPYRIGHT
 
-     Copyright (C) 2002 Kake Pugh.  All Rights Reserved.
+     Copyright (C) 2002-2003 Kake Pugh.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
